

import os
import sys
import shutil
import logging
import pkg_resources
from six.moves.configparser import ConfigParser
from appdirs import user_config_dir


class UpdaterConfig(object):
    _appname = os.path.join('ebs', 'updater')
    _root = os.path.abspath(os.path.dirname(__file__))
    _roots = [_root]
    _config_file = os.path.join(user_config_dir(_appname), 'config.ini')

    def __init__(self):
        if not os.path.exists(os.path.dirname(self._config_file)):
            os.makedirs(os.path.dirname(self._config_file), exist_ok=True)
        if not os.path.exists(self._config_file):
            _ROOT = os.path.abspath(os.path.dirname(__file__))
            shutil.copy(os.path.join(_ROOT, 'default/config.ini'), self._config_file)
        self._config = ConfigParser()
        logging.debug("Reading Config File {}".format(self._config_file))
        self._config.read(self._config_file)
        logging.debug("EBS Linux Node Updater, version {0}".format(self.updater_version))
        self._handlers = {}
        self._domains = {}
        self._package_map = {}

    @property
    def updater_version(self):
        return pkg_resources.get_distribution('ebs-linuxnode-updater').version

    def _write_config(self):
        with open(self._config_file, 'w') as configfile:
            self._config.write(configfile)

    def _check_section(self, section):
        if not self._config.has_section(section):
            self._config.add_section(section)
            self._write_config()

    def _extract_multiline(self, section, option):
        rvals = self._config.get(section, option)
        if '\n' in rvals:
            return [x.strip() for x in rvals.strip().split('\n')]
        else:
            return rvals

    @property
    def domains(self):
        return self._extract_multiline('control', 'domains')

    def install_domain_handler(self, method, handler):
        logging.debug("Installing '{}' as handler for method '{}'".format(handler, method))
        self._handlers[method] = handler

    def register_package_name(self, name, handler):
        logging.debug("Registering '{}' as a package name for handler '{}'".format(name, handler))
        self._package_map[name] = handler

    def __getattr__(self, item):
        if item in self.domains:
            if item not in self._domains.keys():
                method = self._config.get(item, 'method')
                if method not in self._handlers.keys():
                    raise KeyError("'{}' is not a recognized method handler!".format(method))
                self._domains[item] = self._handlers[method](
                    config=self, name=item, **dict(self._config.items(item))
                )
            return self._domains[item]
        elif item in self._package_map.keys():
            return dict(self._config.items(item))
        else:
            raise AttributeError(item)


_config = UpdaterConfig()
sys.modules[__name__] = _config





import logging

from . import log
from . import config

from .domains import pip
pip.install(config)


def updater():
    logging.info("Executing EBS Updater")
    for domain_name in config.domains:
        domain = getattr(config, domain_name)
        domain.update()
    logging.info("Update Check Complete")


if __name__ == '__main__':
    updater()




import os
import logging
import subprocess
from .base import UpdaterDomainBase


class PipDomain(UpdaterDomainBase):
    def __init__(self, **kwargs):
        self._env = kwargs.pop('env')
        self._packages = [x.strip() for x in kwargs.pop('packages').strip().split('\n')]
        super(PipDomain, self).__init__(**kwargs)
        for package in self.packages:
            self._register_package_name(package)

    @property
    def env(self):
        return self._env

    @property
    def packages(self):
        return self._packages

    @property
    def pip_path(self):
        if self.env:
            return os.path.join(self.env, 'bin', 'pip')
        else:
            return 'pip'

    def _execute(self, args):
        cmd = [self.pip_path]
        cmd.extend(args)
        result = subprocess.run(cmd, capture_output=True)
        return result.stdout

    def _get_package_information(self, package):
        result = self._execute(['show', package])
        lines = result.splitlines()
        rv = {}
        for line in lines:
            key, value = line.decode().split(':')
            rv[key.strip()] = value.strip()
        return rv

    def _get_current_version(self, package):
        return self._get_package_information(package)['Version']

    def _get_target_version(self, package):
        return getattr(self._config, package).get('version', 'latest')

    def _update_package(self, package):
        # TODO Remove this when ready for deployment.
        if package != "ebs-linuxnode-updater":
            logging.info("Skipping '{}' package '{}' for update check."
                         "".format(self._name, package))
            return
        logging.debug("Checking '{}' package '{}' for updates"
                      "".format(self._name, package))
        _starting_version = self._get_current_version(package)
        logging.debug("Current version for '{}' : {}"
                      "".format(package, _starting_version))
        logging.debug("Target version for '{}' : {}"
                      "".format(package, self._get_target_version(package)))
        if self._get_target_version(package) == 'latest':
            self._execute(['install', '--upgrade', package])

        _final_version = self._get_current_version(package)
        logging.info("'{}' : {}".format(package, _final_version))
        if _final_version != _starting_version:
            logging.info("Upgraded package '{}' from '{}' to '{}'"
                         "".format(package, _starting_version, _final_version))

    def _update(self):
        for package in self._packages:
            self._update_package(package)


def install(manager):
    manager.install_domain_handler('pip', PipDomain)




import logging


class UpdaterDomainBase(object):
    def __init__(self, **kwargs):
        self._config = kwargs.pop('config')
        self._method = kwargs.pop('method')
        self._name = kwargs.pop('name')

    @property
    def method(self):
        return self._method

    def _register_package_name(self, package):
        self._config.register_package_name(package, self)

    def _update(self):
        raise NotImplementedError

    def update(self):
        logging.debug("Checking domain '{}' for updates".format(self._name))
        self._update()
